// Achtung, Frequenz und Baudrate einstellen
unit SerialPort;

{$mode objfpc}{$H+}
{$goto on}

INTERFACE

Const
  f_CPU = 4000000; //Hertz
  Baud  = 9600;

Procedure UARTInit;
Procedure UARTBusyWait;
Procedure UARTSend(ByteVal: Byte);
Procedure UARTSend(CharVal: Char);inline;
Procedure UARTSendZahl(ByteVal: Byte);
Procedure UARTSendString(InString: String);
//Procedure UARTReceive_ISR;



    
//#############################################################################    
IMPLEMENTATION
uses
  uInterrupts;

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
procedure UARTInit;
Const
  UBRR_VAL   = ((F_CPU+BAUD*8) div (BAUD*16)-1);
begin
  //BaudRate
  UBRRL:= byte(UBRR_VAL);
  UBRRH:= byte(UBRR_VAL div 256);
  //Datenbit, Parität etc
  UCSRC:= (1 shl URSEL) or (1 shl UCSZ2) or (1 shl UCSZ); //8bit, 1Stopbit, keine Parität
  //UART aktivieren
  UCSRB:= (1 shl TXEN)   //TX aktivieren (Senden)
       or (1 shl RXEN)   //RX aktivieren (Empfangen)
       or (1 shl RXCIE); //Interrupt bei Empfang
  //Interrupts erlauben
  InterruptsEnable;

end;

//-------------------------------------------------------------------
Procedure UARTBusyWait;assembler;
label waitBusy;
asm
  waitBusy:  //Warten bis UDR für das nächste Byte bereit ist
    sbis 0x0B,5 //sbis UCSRA,UDRE	
	rjmp waitBusy 
end;

//-------------------------------------------------------------------
Procedure UARTSend(ByteVal: Byte);
begin
  UARTBusyWait;
  UDR:= ByteVal;  
end;

//-------------------------------------------------------------------
Procedure UARTSend(CharVal: Char);inline;
begin
  UARTSend(Byte(CharVal));  
end;

//-------------------------------------------------------------------
Procedure UARTSendZahl(ByteVal: Byte);
var tmp1,tmp10,tmp100: byte;
begin
  tmp1:= ByteVal mod 10;
  ByteVal:= ByteVal div 10;
  tmp10:= ByteVal mod 10;
  tmp100:= ByteVal div 10;
  
  if tmp100 > 0 then begin
    UARTSend(tmp100 or $30);
    UARTSend( tmp10 or $30);
  end else if  tmp10 > 0 then UARTSend(tmp10 or $30);
  
  UARTSend(tmp1 or $30);      
end;

//-------------------------------------------------------------------
Procedure UARTSendString(CharArray: array of Char);
var
  ii:Integer;
begin
  
  for ii:= 0 to length(CharArray)-1 do begin
    UARTBusyWait;
    UDR:= Byte(CharArray[ii]);  
  end;//do

end;

//-------------------------------------------------------------------
Procedure UARTSendString(InString: ShortString);
var
  ii:Integer;
begin
  
  for ii:= 1 to length(InString) do begin
    UARTBusyWait;
    UDR:= Byte(InString[ii]);  
  end;//do

end;
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
Procedure UARTReceive_ISR;Alias: 'USART__RXC_ISR'; Interrupt; Public;
var
tmpByte: Byte; 
begin
  UARTSend(UDR); //Empfangenes Zeichen zurückschicken
end;

//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
end.
